--------------------------------------------------------------------------------
-- Stand Alone Uplink: FPGA Design
-- LIGO document: D060288
-- SubModule LPValid
-- Created   9/8/2006 6:01:09 PM
--------------------------------------------------------------------------------
Library IEEE;
Use IEEE.Std_Logic_1164.all;

entity LPValid is port
   (
     LPV          : out   std_logic;
     CNT          : in    std_logic_vector(25 downto 0);
     MASK         : in    std_logic_vector(31 downto 0);
     START        : in    std_logic_vector(31 downto 0);
     STOP         : in    std_logic_vector(31 downto 0)
   );
end LPValid;
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
architecture Structure of LPValid is

-- Component Declarations

-- Signal Declarations
   signal c    : std_logic_vector(25 downto 0);
   signal b    : std_logic_vector(25 downto 0);
   signal e    : std_logic_vector(25 downto 0);
   signal up   : std_logic;
   signal dn   : std_logic;
   signal lp   : std_logic := '0';
   signal lp0  : std_logic := '1';

begin
   c <= MASK (25 downto 0) and CNT;
   b <= MASK (25 downto 0) and START (31 downto 6);
   e <= MASK (25 downto 0) and STOP (31 downto 6);

   process (b, e, c) is
   begin
      if b = c then
         up <= '1';
      else
         up <= '0';
      end if;
      if e = c then
         dn <= '1';
      else
         dn <= '0';
      end if;
   end process;

   -- RS flip flop
   process (up, dn) is
   begin
      if dn = '1' then
         lpv <= '0';
      elsif up = '1' then
         lpv <= '1';
      end if;
   end process;

end Structure;
--------------------------------------------------------------------------------
