--------------------------------------------------------------------------------
-- Stand Alone Uplink: FPGA Design
-- LIGO document: D060288
-- SubModule MCodeMem
-- Created   9/19/2006 5:19:53 PM
--------------------------------------------------------------------------------
Library IEEE;
Use IEEE.Std_Logic_1164.all;

entity MCodeMem is port
   (
     CLKA          : in    std_logic;
     CLKB          : in    std_logic;
     DINA          : in    std_logic_vector(31 downto 0);
     DOUTA         : out   std_logic_vector(31 downto 0);
     ADDRA         : in    std_logic_vector(8 downto 0);
     WEA           : in    std_logic;
     ENA           : in    std_logic;
     ByteWEA       : in    std_logic_vector(3 downto 0);
     DINB          : in    std_logic_vector(31 downto 0);
     DOUTB         : out   std_logic_vector(31 downto 0);
     ADDRB         : in    std_logic_vector(8 downto 0);
     WEB           : in    std_logic;
     ENB           : in    std_logic;
     ByteWEB       : in    std_logic_vector(3 downto 0)
   );
end MCodeMem;
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
architecture Structure of MCodeMem is

-- Component Declarations
   component bram_512_32
      port (
      clka: IN std_logic;
      dina: IN std_logic_VECTOR(31 downto 0);
      addra: IN std_logic_VECTOR(8 downto 0);
      ena: IN std_logic;
      wea: IN std_logic_VECTOR(3 downto 0);
      douta: OUT std_logic_VECTOR(31 downto 0);
      clkb: IN std_logic;
      dinb: IN std_logic_VECTOR(31 downto 0);
      addrb: IN std_logic_VECTOR(8 downto 0);
      enb: IN std_logic;
      web: IN std_logic_VECTOR(3 downto 0);
      doutb: OUT std_logic_VECTOR(31 downto 0));
   end component;

-- Signal Declarations
   signal wa          : std_logic_vector (3 downto 0);
   signal wb          : std_logic_vector (3 downto 0);

begin
   wa <= ByteWEA when WEA = '1' else B"0000";
   wb <= ByteWEB when WEB = '1' else B"0000";

   -- Dual port RAM
   U_RAM : bram_512_32
        port map (
            clka => clka,
            dina => dina,
            douta => douta,
            addra => addra,
            ena => ena,
            wea => wa,
            clkb => clkb,
            dinb => dinb,
            addrb => addrb,
            enb => enb,
            web => wb,
            doutb => doutb
        );

end Structure;
--------------------------------------------------------------------------------
