--------------------------------------------------------------------------------
-- Stand Alone Uplink: FPGA Design
-- LIGO document: D060288
-- SubModule TX_arbiter
-- Created   9/11/2006 5:09:40 PM
--------------------------------------------------------------------------------
Library IEEE;
Use IEEE.Std_Logic_1164.all;

entity TX_arbiter is port
   (
     -- EMAC interface
     tx_clk            : in    std_logic;
     tx_ack            : in    std_logic;
     tx_data           : out   std_logic_vector(15 downto 0);
     tx_data_valid     : out   std_logic;
     tx_data_valid_msb : out   std_logic;

     -- clock and status
     clk               : in    std_logic;
     reset             : in    std_logic;  -- synchronous
     low_pri_valid     : in    std_logic;
     status            : out   std_logic_vector(15 downto 0);

     -- low priority interface
     LP_en             : in    std_logic;
     LP_d              : in    std_logic_vector(15 downto 0);
     LP_eof            : in    std_logic;
     LP_go             : in    std_logic;
     -- high priority interface A
     A_en              : in    std_logic;
     A_d               : in    std_logic_vector(31 downto 0);
     A_eof             : in    std_logic;
     A_go              : in    std_logic;
     A_flush           : in    std_logic;
     -- high priority interface B
     B_en              : in    std_logic;
     B_d               : in    std_logic_vector(31 downto 0);
     B_eof             : in    std_logic;
     B_go              : in    std_logic;
     B_flush           : in    std_logic;
     -- high priority interface C
     C_en              : in    std_logic;
     C_d               : in    std_logic_vector(31 downto 0);
     C_eof             : in    std_logic;
     C_go              : in    std_logic;
     C_flush           : in    std_logic;
     -- high priority interface D
     D_en              : in    std_logic;
     D_d               : in    std_logic_vector(31 downto 0);
     D_eof             : in    std_logic;
     D_go              : in    std_logic;
     D_flush           : in    std_logic
   );
end TX_arbiter;
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
architecture Structure of TX_arbiter is

-- Type declaration
type tx_arbiter_enum is (IDLE, INIT, TRANSFER, FINI);

-- Component Declarations
   component fifo_1024_18
      port (
         din:             IN std_logic_VECTOR(17 downto 0);
         rd_clk:          IN std_logic;
         rd_en:           IN std_logic;
         rst:             IN std_logic;
         wr_clk:          IN std_logic;
         wr_en:           IN std_logic;
         dout:            OUT std_logic_VECTOR(17 downto 0);
         empty:           OUT std_logic;
         full:            OUT std_logic;
         overflow:        OUT std_logic;
         underflow:       OUT std_logic
      );
   end component;
   component fifo_2048_18
      port (
         din:             IN std_logic_VECTOR(17 downto 0);
         rd_clk:          IN std_logic;
         rd_en:           IN std_logic;
         rst:             IN std_logic;
         wr_clk:          IN std_logic;
         wr_en:           IN std_logic;
         dout:            OUT std_logic_VECTOR(17 downto 0);
         empty:           OUT std_logic;
         full:            OUT std_logic;
         overflow:        OUT std_logic;
         underflow:       OUT std_logic
      );
   end component;
   component fifo_1024_36_18
      port (
         din:             IN std_logic_VECTOR(35 downto 0);
         rd_clk:          IN std_logic;
         rd_en:           IN std_logic;
         rst:             IN std_logic;
         wr_clk:          IN std_logic;
         wr_en:           IN std_logic;
         dout:            OUT std_logic_VECTOR(17 downto 0);
         empty:           OUT std_logic;
         full:            OUT std_logic;
         overflow:        OUT std_logic;
         underflow:       OUT std_logic
      );
   end component;
   component fifo_4096_18
      port (
         din:             IN std_logic_VECTOR(17 downto 0);
         rd_clk:          IN std_logic;
         rd_en:           IN std_logic;
         rst:             IN std_logic;
         wr_clk:          IN std_logic;
         wr_en:           IN std_logic;
         dout:            OUT std_logic_VECTOR(17 downto 0);
         empty:           OUT std_logic;
         full:            OUT std_logic;
         overflow:        OUT std_logic;
         underflow:       OUT std_logic
      );
   end component;
   component fifo_8192_18
      port (
         din:             IN std_logic_VECTOR(17 downto 0);
         rd_clk:          IN std_logic;
         rd_en:           IN std_logic;
         rst:             IN std_logic;
         wr_clk:          IN std_logic;
         wr_en:           IN std_logic;
         dout:            OUT std_logic_VECTOR(17 downto 0);
         empty:           OUT std_logic;
         full:            OUT std_logic;
         overflow:        OUT std_logic;
         underflow:       OUT std_logic
      );
   end component;
   -- declare go block
   component TX_FIFO_ready
      port (
         reset:           in  std_logic;
         wr_clk:          in  std_logic;
         go:              in  std_logic;
         rd_clk:          in  std_logic;
         ready:           out std_logic;
         ack:             in  std_logic;
         err:             out std_logic
      );
   end component TX_FIFO_ready;

-- Signal Declarations
   signal reset_r : std_logic_vector(4 downto 0);

   signal LP_in : std_logic_vector(17 downto 0);
   signal LP_out : std_logic_vector(17 downto 0);
   signal LP_rd : std_logic;
   signal LP_empty : std_logic;
   signal LP_full : std_logic;
   signal LP_overflow : std_logic;
   signal LP_underflow : std_logic;
   signal A_reset : std_logic;
   signal A_in : std_logic_vector(35 downto 0);
   signal A_out : std_logic_vector(17 downto 0);
   signal A_rd : std_logic;
   signal A_empty : std_logic;
   signal A_full : std_logic;
   signal A_overflow : std_logic;
   signal A_underflow : std_logic;
   signal B_reset : std_logic;
   signal B_in : std_logic_vector(35 downto 0);
   signal B_out : std_logic_vector(17 downto 0);
   signal B_rd : std_logic;
   signal B_empty : std_logic;
   signal B_full : std_logic;
   signal B_overflow : std_logic;
   signal B_underflow : std_logic;
   signal C_reset : std_logic;
   signal C_in : std_logic_vector(35 downto 0);
   signal C_out : std_logic_vector(17 downto 0);
   signal C_rd : std_logic;
   signal C_empty : std_logic;
   signal C_full : std_logic;
   signal C_overflow : std_logic;
   signal C_underflow : std_logic;
   signal D_reset : std_logic;
   signal D_in : std_logic_vector(35 downto 0);
   signal D_out : std_logic_vector(17 downto 0);
   signal D_rd : std_logic;
   signal D_empty : std_logic;
   signal D_full : std_logic;
   signal D_overflow : std_logic;
   signal D_underflow : std_logic;

   signal FIFO_sel : std_logic_vector (2 downto 0);
   signal sel_out : std_logic_vector (17 downto 0);
   signal LP_ready : std_logic;
   signal A_ready : std_logic;
   signal B_ready : std_logic;
   signal C_ready : std_logic;
   signal D_ready : std_logic;
   signal LP_ack : std_logic;
   signal A_ack : std_logic;
   signal B_ack : std_logic;
   signal C_ack : std_logic;
   signal D_ack : std_logic;
   signal LP_go_err : std_logic;
   signal A_go_err : std_logic;
   signal B_go_err : std_logic;
   signal C_go_err : std_logic;
   signal D_go_err : std_logic;
   signal transmit : std_logic;
   signal flush : std_logic;
   signal empty : std_logic;
   signal transmit_err : std_logic;
   signal FIFO_read : std_logic;
   signal stop_read : std_logic;
   signal status_1: std_logic_vector(15 downto 0);
   signal status_2: std_logic_vector(15 downto 0);
   signal status_3: std_logic_vector(15 downto 0);
   signal tx_arbiter_state : tx_arbiter_enum := IDLE;

begin
   -- extend reset signal
   process (tx_clk, reset) is
   begin
      if (reset = '1') then
         reset_r <= "11111";
      elsif rising_edge (tx_clk) then
         reset_r <= '0' & reset_r(4) & reset_r(3) & reset_r(2) & reset_r(1);
      end if;
   end process;
   A_reset <= reset_r(0) or A_flush;
   B_reset <= reset_r(0) or B_flush;
   C_reset <= reset_r(0) or C_flush;
   D_reset <= reset_r(0) or D_flush;


   -- append input vector to 18 bits
   LP_in <= LP_d & LP_eof & '0';
   -- append input vector to 36 bits; FIFO reads MSW first
   A_in <= A_d(31 downto 16) & B"00" & A_d(15 downto 0) & A_eof & '0';
   B_in <= B_d(31 downto 16) & B"00" & B_d(15 downto 0) & B_eof & '0';
   C_in <= C_d(31 downto 16) & B"00" & C_d(15 downto 0) & C_eof & '0';
   D_in <= D_d(31 downto 16) & B"00" & D_d(15 downto 0) & D_eof & '0';

   -- declare FIFOs
   LP_tx_fifo : fifo_2048_18
      port map (
         rst => reset_r(0),
         rd_clk => tx_clk,
         rd_en => LP_rd,
         dout => LP_out,
         wr_clk => clk,
         wr_en => LP_en,
         din => LP_in,
         empty => LP_empty,
         full => LP_full,
         overflow => LP_overflow,
         underflow => LP_underflow
      );
   A_tx_fifo : fifo_1024_36_18
      port map (
         rst => A_reset,
         rd_clk => tx_clk,
         rd_en => A_rd,
         dout => A_out,
         wr_clk => clk,
         wr_en => A_en,
         din => A_in,
         empty => A_empty,
         full => A_full,
         overflow => A_overflow,
         underflow => A_underflow
      );
   B_tx_fifo : fifo_1024_36_18
      port map (
         rst => B_reset,
         rd_clk => tx_clk,
         rd_en => B_rd,
         dout => B_out,
         wr_clk => clk,
         wr_en => B_en,
         din => B_in,
         empty => B_empty,
         full => B_full,
         overflow => B_overflow,
         underflow => B_underflow
      );
   C_tx_fifo : fifo_1024_36_18
      port map (
         rst => C_reset,
         rd_clk => tx_clk,
         rd_en => C_rd,
         dout => C_out,
         wr_clk => clk,
         wr_en => C_en,
         din => C_in,
         empty => C_empty,
         full => C_full,
         overflow => C_overflow,
         underflow => C_underflow
      );
   D_tx_fifo : fifo_1024_36_18
      port map (
         rst => D_reset,
         rd_clk => tx_clk,
         rd_en => D_rd,
         dout => D_out,
         wr_clk => clk,
         wr_en => D_en,
         din => D_in,
         empty => D_empty,
         full => D_full,
         overflow => D_overflow,
         underflow => D_underflow
      );

   -- implement output mux
   output_mux: process (LP_out, LP_empty, A_out, B_out, C_out, D_out,
                        A_flush, B_flush, C_flush, D_flush,
                        A_empty, B_empty, C_empty, D_empty, FIFO_sel) is
   begin
      case FIFO_sel is
         when B"000" =>
            sel_out <= A_out;
            flush <= A_flush;
            empty <= A_empty;
         when B"001" =>
            sel_out <= B_out;
            flush <= B_flush;
            empty <= B_empty;
         when B"010" =>
            sel_out <= C_out;
            flush <= C_flush;
            empty <= C_empty;
         when B"011" =>
            sel_out <= D_out;
            flush <= D_flush;
            empty <= D_empty;
         when others =>
            sel_out <= LP_out;
            flush <= '0';
            empty <= LP_empty;
      end case;
   end process output_mux;

   -- implement 'go' logic
   LP_tx_go: TX_FIFO_ready
      port map (
         reset => reset_r(0),
         wr_clk => clk,
         go => LP_go,
         rd_clk => tx_clk,
         ready => LP_ready,
         ack => LP_ack,
         err => LP_go_err
      );
   A_tx_go: TX_FIFO_ready
      port map (
         reset => A_reset,
         wr_clk => clk,
         go => A_go,
         rd_clk => tx_clk,
         ready => A_ready,
         ack => A_ack,
         err => A_go_err
      );
   B_tx_go: TX_FIFO_ready
      port map (
         reset => B_reset,
         wr_clk => clk,
         go => B_go,
         rd_clk => tx_clk,
         ready => B_ready,
         ack => B_ack,
         err => B_go_err
      );
   C_tx_go: TX_FIFO_ready
      port map (
         reset => C_reset,
         wr_clk => clk,
         go => C_go,
         rd_clk => tx_clk,
         ready => C_ready,
         ack => C_ack,
         err => C_go_err
      );
   D_tx_go: TX_FIFO_ready
      port map (
         reset => D_reset,
         wr_clk => clk,
         go => D_go,
         rd_clk => tx_clk,
         ready => D_ready,
         ack => D_ack,
         err => D_go_err
      );

   -- tx arbiter finite-state machine
   tx_arbiter_FSM: process (tx_clk, tx_arbiter_state, flush, empty) is
   begin
      if rising_edge (tx_clk) then
         if reset_r(0) = '1' then
            tx_arbiter_state <= IDLE;
            transmit <= '0';
            stop_read <= '0';
            A_ack <= '0';
            B_ack <= '0';
            C_ack <= '0';
            D_ack <= '0';
            LP_ack <= '0';
         else
            case tx_arbiter_state is
               when IDLE =>
                  transmit <= '0';
                  stop_read <= '0';
                  if A_ready = '1' then
                     FIFO_sel <= B"000";
                     transmit <= '1';
                     A_ack <= '1';
                     tx_arbiter_state <= INIT;
                  elsif B_ready = '1' then
                     FIFO_sel <= B"001";
                     transmit <= '1';
                     B_ack <= '1';
                     tx_arbiter_state <= INIT;
                  elsif C_ready = '1' then
                     FIFO_sel <= B"010";
                     transmit <= '1';
                     C_ack <= '1';
                     tx_arbiter_state <= INIT;
                  elsif D_ready = '1' then
                     FIFO_sel <= B"011";
                     transmit <= '1';
                     D_ack <= '1';
                     tx_arbiter_state <= INIT;
                  elsif (LP_ready = '1') and (low_pri_valid = '1') then
                     FIFO_sel <= B"100";
                     transmit <= '1';
                     LP_ack <= '1';
                     tx_arbiter_state <= INIT;
                  end if;
               when INIT =>
                  A_ack <= '0';
                  B_ack <= '0';
                  C_ack <= '0';
                  D_ack <= '0';
                  LP_ack <= '0';
                  if (flush = '1') or (empty = '1') then
                     tx_arbiter_state <= IDLE;
                     stop_read <= '1';
                     transmit <= '0';
                  else
                     tx_arbiter_state <= TRANSFER;
                  end if;
               when TRANSFER =>
                  if (flush = '1') or (empty = '1') then
                     tx_arbiter_state <= IDLE;
                     stop_read <= '1';
                     transmit <= '0';
                  elsif sel_out(1) = '1' then
                     tx_arbiter_state <= FINI;
                  end if;
               when FINI =>
                  stop_read <= '1';
                  transmit <= '0';
                  tx_arbiter_state <= IDLE;
            end case;
         end if;
      end if;
   end process tx_arbiter_FSM;

   -- FIFO read enable logic
   FIFO_read_sel: process (FIFO_sel, FIFO_read) is
   begin
      case FIFO_sel is
         when B"000" =>
            A_rd <= FIFO_read;
            B_rd <= '0';
            C_rd <= '0';
            D_rd <= '0';
            LP_rd <= '0';
         when B"001" =>
            A_rd <= '0';
            B_rd <= FIFO_read;
            C_rd <= '0';
            D_rd <= '0';
            LP_rd <= '0';
         when B"010" =>
            A_rd <= '0';
            B_rd <= '0';
            C_rd <= FIFO_read;
            D_rd <= '0';
            LP_rd <= '0';
         when B"011" =>
            A_rd <= '0';
            B_rd <= '0';
            C_rd <= '0';
            D_rd <= FIFO_read;
            LP_rd <= '0';
         when others =>
            A_rd <= '0';
            B_rd <= '0';
            C_rd <= '0';
            D_rd <= '0';
            LP_rd <= FIFO_read;
      end case;
   end process FIFO_read_sel;
   -- FIFO read state machine
   FIFO_read_FSM: process (reset_r, stop_read, tx_ack, flush, empty) is
   begin
      if (reset_r(0) = '1') or (stop_read = '1') or (flush = '1') or
         (empty = '1') then
         FIFO_read <= '0';
      elsif (tx_ack = '1') then
         FIFO_read <= '1';
      end if;
   end process FIFO_read_FSM;

   -- EMAC interface
   tx_data <= sel_out(17 downto 2);
   tx_data_valid <= transmit;
   tx_data_valid_msb <= '1';  -- we always write full words

   -- status
   transmit_err <= '1' when ((flush = '1') or (empty = '1')) and
                            ((tx_arbiter_state = TRANSFER) or
                             (tx_arbiter_state = INIT)) else '0';
   status_1 <= transmit_err &
               LP_go_err & LP_underflow & LP_overflow &
               D_go_err & D_underflow & D_overflow &
               C_go_err & C_underflow & C_overflow &
               B_go_err & B_underflow & B_overflow &
               A_go_err & A_underflow & A_overflow;
   -- now resync to clk
   status_2 <= status_1 when rising_edge (clk);
   status_3 <= status_3 when rising_edge (clk);
   status <= status_2 and not status_3;

end Structure;
--------------------------------------------------------------------------------
